package net.kierenb.mapcast.api;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.Serializable;
import java.util.Set;

import net.kierenb.mapcast.ReplicatedMap;
import net.kierenb.mapcast.ReplicatedMapServer;
import net.kierenb.mapcast.replication.EntryDao;
import net.kierenb.mapcast.util.HttpResource;

import org.apache.log4j.lf5.util.StreamUtils;
import org.simpleframework.http.Request;
import org.simpleframework.http.Response;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class MapResource implements HttpResource {

    public static final String URL = "/maps/";
    public static final Integer KEY_LIMIT = 50;

    private ReplicatedMapServer server;
    private EntryDao entryDao;

    public MapResource(ReplicatedMapServer server, EntryDao entryDao) {
        this.server = server;
        this.entryDao = entryDao;
    }

    @Override
    public String getUrl() {
        return URL;
    }

    @Override
    public void handle(Request req, Response resp) throws IOException {
        String path = req.getPath().getPath().substring(URL.length());
        if (path.length() == 0) {
            if (!"GET".equals(req.getMethod())) {
                resp.setCode(405);
                return;
            }
            
            resp.set("content-type", "text/html");
            PrintStream out = resp.getPrintStream();
            out.print("<html><body>");
            for (String map : entryDao.getMaps()) {
                if (!ReplicatedMapServer.LAG_MAP_NAME.equals(map)) {
                    out.print("<a href=\"" + map + "/\">" + map + "</a><br />");
                }
            }
            out.print("</body></html>");
        }
        else {
            int indexOfSlash = path.indexOf('/');
            if (indexOfSlash >= 0) {
                String map = path.substring(0, indexOfSlash);
                if (!ReplicatedMapServer.LAG_MAP_NAME.equals(map)) {
                    handleMap(req, resp, map, path.substring(indexOfSlash + 1));
                    return;
                }
            }

            if (!"GET".equals(req.getMethod())) {
                resp.setCode(405);
            }
            else {
                resp.setCode(404);
            }
        }
    }

    private void handleMap(Request req, Response resp, String mapName, String keyName) throws IOException {
        // the value serializer doesn't matter here as we only get raw values
        ReplicatedMap<String, ? extends Serializable> map = server.getMap(mapName);

        if (keyName.length() == 0) {
            if (!"GET".equals(req.getMethod())) {
                resp.setCode(405);
                return;
            }
            
            resp.set("content-type", "text/html");
            PrintStream out = resp.getPrintStream();
            out.print("<html><body>");
            String keyFrom = req.getParameter("keyFrom");
            Set<String> keys;
            if (keyFrom != null) {
                 keys = map.keySet(keyFrom, KEY_LIMIT);
            }
            else {
                keys = map.keySet(KEY_LIMIT);
            }
            for (String key : keys) {
                out.print("<a href=\"" + key + "\">" + key + "</a><br />");
                keyFrom = key;
            }
            if (keys.size() >= KEY_LIMIT) {
                out.print("---<br /><a href=\"?keyFrom=" + keyFrom + "\">More...</a><br />");
            }
            out.print("</body></html>");
        }
        else {
            if ("GET".equals(req.getMethod())) {
                if (map.containsKey(keyName)) {
                    OutputStream out = resp.getPrintStream();
                    out.write(map.getRaw(keyName));
                }
                else {
                    resp.setCode(404);
                }
            }
            else if ("PUT".equals(req.getMethod())) {
                byte[] bytes = StreamUtils.getBytes(req.getInputStream());
                map.putRaw(keyName, bytes);
            }
            else {
                resp.setCode(405);
            }
        }
    }

}
