package net.kierenb.mapcast.replication;

import java.util.Arrays;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class ReplicatedMapEntry {
    private String map;
    private String key;
    private byte[] value;
    private Long deletedTime;
    private String lastUpdatedNode;
    private Long lastUpdatedTime;

    public ReplicatedMapEntry(String map, String key, byte[] value, Long deletedTime, String lastUpdatedNode, Long lastUpdatedTime) {
        this.map = map;
        this.key = key;
        this.value = value;
        this.deletedTime = deletedTime;
        this.lastUpdatedNode = lastUpdatedNode;
        this.lastUpdatedTime = lastUpdatedTime;
    }

    public String getMap() {
        return map;
    }

    public String getKey() {
        return key;
    }

    public byte[] getValue() {
        return value;
    }

    public Long getDeletedTime() {
        return deletedTime;
    }

    public String getLastUpdatedNode() {
        return lastUpdatedNode;
    }

    public Long getLastUpdatedTime() {
        return lastUpdatedTime;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((deletedTime == null) ? 0 : deletedTime.hashCode());
        result = prime * result + ((key == null) ? 0 : key.hashCode());
        result = prime * result + ((lastUpdatedNode == null) ? 0 : lastUpdatedNode.hashCode());
        result = prime * result + ((lastUpdatedTime == null) ? 0 : lastUpdatedTime.hashCode());
        result = prime * result + ((map == null) ? 0 : map.hashCode());
        result = prime * result + Arrays.hashCode(value);
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (getClass() != obj.getClass()) return false;
        ReplicatedMapEntry other = (ReplicatedMapEntry)obj;
        if (deletedTime == null) {
            if (other.deletedTime != null) return false;
        }
        else if (!deletedTime.equals(other.deletedTime)) return false;
        if (key == null) {
            if (other.key != null) return false;
        }
        else if (!key.equals(other.key)) return false;
        if (lastUpdatedNode == null) {
            if (other.lastUpdatedNode != null) return false;
        }
        else if (!lastUpdatedNode.equals(other.lastUpdatedNode)) return false;
        if (lastUpdatedTime == null) {
            if (other.lastUpdatedTime != null) return false;
        }
        else if (!lastUpdatedTime.equals(other.lastUpdatedTime)) return false;
        if (map == null) {
            if (other.map != null) return false;
        }
        else if (!map.equals(other.map)) return false;
        if (!Arrays.equals(value, other.value)) return false;
        return true;
    }

}
