package net.kierenb.mapcast.replication;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLSyntaxErrorException;
import java.util.HashMap;
import java.util.Map;

import net.kierenb.mapcast.sql.ResultTransformer;
import net.kierenb.mapcast.sql.SqlUtil;
import net.kierenb.mapcast.sql.StatementPreparator;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class StateDao {

    private SqlUtil sqlUtil;

    public StateDao(SqlUtil sqlUtil) {
        this.sqlUtil = sqlUtil;

        try {
            sqlUtil.executeUpdate("CREATE TABLE state (node VARCHAR(40), time BIGINT, PRIMARY KEY(node))");
        }
        catch (RuntimeException e) {
            // the table already exists
            if (!(e.getCause() instanceof SQLSyntaxErrorException)) {
                throw e;
            }
        }
    }

    public Map<String, Long> getState() {
        return sqlUtil.executeQuery("SELECT node, time FROM state", new ResultTransformer<Map<String, Long>>() {
            @Override
            public Map<String, Long> transform(ResultSet rs) throws SQLException {
                Map<String, Long> state = new HashMap<String, Long>();
                while (rs.next()) {
                    String node = rs.getString("node");
                    Long time = rs.getLong("time");
                    state.put(node, time);
                }
                return state;
            }
        });
    }

    public void updateState(final String node, final Long time) {
        sqlUtil.executeUpdate("MERGE INTO state USING (VALUES(?, CAST(? AS BIGINT))) AS new(node, time) ON state.node = new.node WHEN MATCHED THEN UPDATE SET state.time = new.time WHEN NOT MATCHED THEN INSERT VALUES new.node, new.time", new StatementPreparator() {
            @Override
            public void prepare(PreparedStatement stmt) throws SQLException {
                int i = 1;
                stmt.setString(i++, node);
                stmt.setLong(i++, time);
            }
        });
    }

}
