package net.kierenb.mapcast.serializer;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class BinaryValueSerializer<V extends Serializable> implements ValueSerializer<V> {

    @SuppressWarnings("unchecked")
    @Override
    public V toObject(byte[] bytes) {
        try {
            ByteArrayInputStream inByte = new ByteArrayInputStream(bytes);
            ObjectInputStream inObj = new ObjectInputStream(inByte);
            return (V)inObj.readObject();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
        catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public byte[] fromObject(V obj) {
        try {
            ByteArrayOutputStream outByte = new ByteArrayOutputStream();
            ObjectOutputStream outObj = new ObjectOutputStream(outByte);
            outObj.writeObject(obj);
            outObj.flush();
            outObj.close();
            return outByte.toByteArray();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

}
