package net.kierenb.mapcast.serializer;

import java.beans.ExceptionListener;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Serializable;

import org.apache.log4j.Logger;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class XmlValueSerializer<V extends Serializable> implements ValueSerializer<V> {
    private static final Logger log = Logger.getLogger(XmlValueSerializer.class);

    @SuppressWarnings("unchecked")
    @Override
    public V toObject(byte[] bytes) {
        ByteArrayInputStream inByte = new ByteArrayInputStream(bytes);
        XMLDecoder inObj = new XMLDecoder(inByte);
        inObj.setExceptionListener(new ExceptionListener() {
            @Override
            public void exceptionThrown(Exception e) {
                log.warn("Non fatal exception decoding object", e);
            }
        });
        inObj.close();
        return (V)inObj.readObject();
    }

    @Override
    public byte[] fromObject(V obj) {
        ByteArrayOutputStream outByte = new ByteArrayOutputStream();
        XMLEncoder outObj = new XMLEncoder(outByte);
        outObj.setExceptionListener(new ExceptionListener() {
            @Override
            public void exceptionThrown(Exception e) {
                log.warn("Non fatal exception decoding object", e);
            }
        });
        outObj.writeObject(obj);
        outObj.flush();
        outObj.close();
        return outByte.toByteArray();
    }

}
