package net.kierenb.mapcast.util;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLSyntaxErrorException;
import java.util.UUID;

import net.kierenb.mapcast.sql.ResultTransformer;
import net.kierenb.mapcast.sql.SqlUtil;
import net.kierenb.mapcast.sql.StatementPreparator;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class ConfigDao {
    private static final String CONFIG_NODE = "node";

    private SqlUtil sqlUtil;

    public ConfigDao(SqlUtil sqlUtil) {
        this.sqlUtil = sqlUtil;

        try {
            sqlUtil.executeUpdate("CREATE TABLE config (key VARCHAR(20), value VARCHAR(255), PRIMARY KEY(key))");
        }
        catch (RuntimeException e) {
            // the table already exists
            if (!(e.getCause() instanceof SQLSyntaxErrorException)) {
                throw e;
            }
        }
    }

    public String get(final String key) {
        return sqlUtil.executeQuery("SELECT value FROM config WHERE key=?", new StatementPreparator() {
            @Override
            public void prepare(PreparedStatement stmt) throws SQLException {
                int i = 1;
                stmt.setString(i++, key);
            }
        }, new ResultTransformer<String>() {
            @Override
            public String transform(ResultSet rs) throws SQLException {
                if (rs.next()) {
                    return rs.getString("value");
                }
                return null;
            }
        });
    }

    public void set(final String key, final String value) {
        sqlUtil.executeUpdate("MERGE INTO config USING (VALUES(?, ?)) AS new(key,value) ON config.key = new.key WHEN MATCHED THEN UPDATE SET config.value = new.value WHEN NOT MATCHED THEN INSERT VALUES new.key, new.value", new StatementPreparator() {
            @Override
            public void prepare(PreparedStatement stmt) throws SQLException {
                int i = 1;
                stmt.setString(i++, key);
                stmt.setString(i++, value);
            }
        });
    }

    public String getNode() {
        String node = get(CONFIG_NODE);
        if (node == null) {
            node = UUID.randomUUID().toString();
            set(CONFIG_NODE, node);
        }
        return node;
    }

}
