package net.kierenb.mapcast.util;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public abstract class PeriodicTask implements Runnable {

    private String threadName;
    private Integer pollingPeriod;
    private Thread thread;

    public PeriodicTask(String threadName, Integer pollingPeriod) {
        this.threadName = threadName;
        this.pollingPeriod = pollingPeriod;
    }

    public void start() {
        thread = new Thread(this, threadName);
        thread.setDaemon(true);
        thread.start();
    }

    @Override
    public void run() {
        while (!thread.isInterrupted()) {
            doTask();
            try {
                Thread.sleep(pollingPeriod);
            }
            catch (InterruptedException e) {
                thread.interrupt();
            }
        }
    }

    public abstract void doTask();

}
