package net.kierenb.mapcast.util;

import java.util.concurrent.atomic.AtomicLong;

/**
 * Copyright 2012 Kieren Beckett
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 * 
 * @author mail@kierenb.net
 */
public class TimeBasedId {
    private static int BITS_PER_MS = 16;

    private AtomicLong id = new AtomicLong(System.currentTimeMillis());

    public long getId() {
        // we want to use the current millisecond as the id, but we also bit shift the time and append a unique number
        // within each ms to make what we're returning unique (within this JVM)

        long previousId = id.get();
        long currentTime = System.currentTimeMillis();
        while (currentTime != previousId >> BITS_PER_MS) {
            if (id.compareAndSet(previousId, currentTime << BITS_PER_MS)) {
                return currentTime << BITS_PER_MS;
            }
            previousId = id.get();
            currentTime = System.currentTimeMillis();
        }
        return id.incrementAndGet();
    }

}
